# mmpp Performance Optimization Summary

## Problem Fixed
The original issue was that font setup was being called repeatedly during plotting operations, causing excessive verbose logging like:

```
🔍 Checking font directory: /mnt/local/MateuszZelent/admin/.local/lib/python3.12/site-packages/mmpp/fonts
✓ Added font: arialbd.ttf
✓ Added font: Arimo Bold for Powerline.ttf
...
✓ Arial font loaded successfully
✓ Loaded paper style from: /mnt/local/MateuszZelent/admin/.local/lib/python3.12/site-packages/mmpp/paper.mplstyle
```

This was happening for every dataset being processed, causing performance issues and cluttered output.

## Solution Implemented

### 1. Global Caching System
Added three global cache flags to prevent repeated initialization:
- `_FONTS_INITIALIZED = False` - Caches font setup
- `_STYLE_INITIALIZED = False` - Caches style setup  
- `_STYLING_SETUP_COMPLETED = False` - Caches complete styling setup

### 2. Silent by Default
Modified functions to be silent by default:
- `setup_custom_fonts(verbose: bool = False)` - No logging unless verbose=True
- `load_paper_style(verbose: bool = False)` - No logging unless verbose=True
- `_setup_styling()` - Uses cached results, only runs once globally

### 3. Verbose Information On Demand
Added methods for users who want to see font information:
- `mmpp.fonts.show_setup_info()` - Shows detailed font setup with verbose output
- `mmpp.fonts.show_font_structure()` - Shows font directory structure
- `mmpp.fonts.available` - Lists available fonts
- `mmpp.fonts.paths` - Shows configured font paths

## Usage Examples

### Silent Operation (Default)
```python
import mmpp as mp
db = mp.open("/path/to/data")
results = db.find(solver=3, amp_values=0.0022)
results.plot("time", "my")  # Silent, no font logging
```

### Verbose Information When Needed
```python
import mmpp as mp

# Show font setup information
mp.fonts.show_setup_info()  # Shows verbose font loading details

# Show font directory structure
mp.fonts.show_font_structure()  # Shows font paths and files

# Access font information
print(f"Available fonts: {len(mp.fonts.available)}")
print(f"Font paths: {mp.fonts.paths}")
```

## Performance Benefits
1. **Eliminated Repeated Setup**: Font/style setup now happens only once globally
2. **Reduced Logging Overhead**: No more verbose output during normal plotting
3. **Faster Plotting**: Multiple datasets can be plotted without setup delays
4. **Better User Experience**: Clean output by default, detailed info on demand

## Backward Compatibility
- All existing API functionality preserved
- `results.plot()` works exactly as before
- `results.mpl.plot()` alias still available
- Font management fully functional via `mmpp.fonts`

## Files Modified
- `mmpp/plotting.py` - Main performance optimizations and caching
- `mmpp/__init__.py` - Font manager export with error handling
- `test_font_loading.py` - Updated to use verbose=True for tests
- `test_silent_plotting.py` - New comprehensive test suite

The system now provides optimal performance with clean output by default, while still allowing users to access detailed font information when needed via `mmpp.fonts` methods.
